@@ -1,42 +1,43 @@
-from ethereum import blocks
-from logging import StreamHandler
-from IPython.core import ultratb
-import monkeypatches
+# -*- coding: utf8 -*-
+import copy
 import json
 import os
 import signal
 import sys
-import copy
+from logging import StreamHandler
 from uuid import uuid4
+
 import click
-from click import BadParameter
-import gevent
-from gevent.event import Event
-import rlp
-from devp2p.service import BaseService
-from devp2p.peermanager import PeerManager
-from devp2p.discovery import NodeDiscovery
-from devp2p.app import BaseApp
-import eth_protocol
-from eth_service import ChainService
-from console_service import Console
-from ethereum.blocks import Block
 import ethereum.slogging as slogging
+import gevent
+import rlp
+from click import BadParameter
+from devp2p.app import BaseApp
+from devp2p.discovery import NodeDiscovery
+from devp2p.peermanager import PeerManager
+from devp2p.service import BaseService
+from ethereum import blocks
+from ethereum.blocks import Block
+from gevent.event import Event
+
 import config as konfig
+import eth_protocol
+import utils
+from accounts import AccountsService, Account
+from console_service import Console
 from db_service import DBService
-from jsonrpc import JSONRPCServer
+from eth_service import ChainService
+from jsonrpc import JSONRPCServer, IPCRPCServer
 from pow_service import PoWService
-from accounts import AccountsService, Account, DEFAULT_COINBASE
 from pyethapp import __version__
 from pyethapp.profiles import PROFILES, DEFAULT_PROFILE
-from pyethapp.utils import update_config_from_genesis_json, merge_dict
-import utils
+from pyethapp.utils import merge_dict, load_contrib_services, FallbackChoice, enable_greenlet_debugger
+
 
 log = slogging.get_logger('app')
 
-
 services = [DBService, AccountsService, NodeDiscovery, PeerManager, ChainService, PoWService,
-            JSONRPCServer, Console]
+            JSONRPCServer, IPCRPCServer, Console]
 
 
 class EthApp(BaseApp):
@@ -48,17 +49,25 @@ class EthApp(BaseApp):
     default_config = dict(BaseApp.default_config)
     default_config['client_version_string'] = client_version_string
     default_config['post_app_start_callback'] = None
+    script_globals = {}
 
 
+# TODO: Remove `profile` fallbacks in 1.4 or so
 # Separators should be underscore!
-@click.group(help='Welcome to {}  version: {}'.format(EthApp.client_name, EthApp.client_version))
-@click.option('--profile', type=click.Choice(PROFILES.keys()), default=DEFAULT_PROFILE,
-              help="Configuration profile.", show_default=True)
-@click.option('alt_config', '--Config', '-C', type=click.File(), help='Alternative config file')
+@click.group(help='Welcome to {} {}'.format(EthApp.client_name, EthApp.client_version))
+@click.option('--profile', type=FallbackChoice(
+                  PROFILES.keys(),
+                  {'frontier': 'livenet', 'morden': 'testnet'},
+                  "PyEthApp's configuration profiles have been renamed to "
+                  "'livenet' and 'testnet'. The previous values 'frontier' and "
+                  "'morden' will be removed in a future update."),
+              default=DEFAULT_PROFILE, help="Configuration profile.", show_default=True)
+@click.option('alt_config', '--Config', '-C', type=str, callback=konfig.validate_alt_config_file,
+              help='Alternative config file')
 @click.option('config_values', '-c', multiple=True, type=str,
               help='Single configuration parameters (<param>=<value>)')
-@click.option('-d', '--data-dir', multiple=False, type=str,
-              help='data directory')
+@click.option('alt_data_dir', '-d', '--data-dir', multiple=False, type=str,
+              help='data directory', default=konfig.default_data_dir, show_default=True)
 @click.option('-l', '--log_config', multiple=False, type=str, default=":info",
               help='log_config string: e.g. ":info,eth:debug', show_default=True)
 @click.option('--log-json/--log-no-json', default=False,
@@ -73,14 +82,14 @@ class EthApp(BaseApp):
               help='Unlock an account (prompts for password)')
 @click.option('--password', type=click.File(), help='path to a password file')
 @click.pass_context
-def app(ctx, profile, alt_config, config_values, data_dir, log_config, bootstrap_node, log_json,
+def app(ctx, profile, alt_config, config_values, alt_data_dir, log_config, bootstrap_node, log_json,
         mining_pct, unlock, password, log_file):
-
     # configure logging
     slogging.configure(log_config, log_json=log_json, log_file=log_file)
 
     # data dir default or from cli option
-    data_dir = data_dir or konfig.default_data_dir
+    alt_data_dir = os.path.expanduser(alt_data_dir)
+    data_dir = alt_data_dir or konfig.default_data_dir
     konfig.setup_data_dir(data_dir)  # if not available, sets up data_dir and required config
     log.info('using data in', path=data_dir)
 
@@ -88,35 +97,55 @@ def app(ctx, profile, alt_config, config_values, data_dir, log_config, bootstrap
     # config files only contain required config (privkeys) and config different from the default
     if alt_config:  # specified config file
         config = konfig.load_config(alt_config)
+        if not config:
+            log.warning('empty config given. default config values will be used')
     else:  # load config from default or set data_dir
         config = konfig.load_config(data_dir)
 
     config['data_dir'] = data_dir
 
+    # Store custom genesis to restore if overridden by profile value
+    genesis_from_config_file = config.get('eth', {}).get('genesis')
+
+    # Store custom bootstrap_nodes to restore them overridden by profile value
+    bootstrap_nodes_from_config_file = config.get('discovery', {}).get('bootstrap_nodes')
+
     # add default config
     konfig.update_config_with_defaults(config, konfig.get_default_config([EthApp] + services))
 
-    log.DEV("Move to EthApp.default_config")
     konfig.update_config_with_defaults(config, {'eth': {'block': blocks.default_config}})
 
     # Set config values based on profile selection
     merge_dict(config, PROFILES[profile])
 
+    if genesis_from_config_file:
+        # Fixed genesis_hash taken from profile must be deleted as custom genesis loaded
+        del config['eth']['genesis_hash']
+        config['eth']['genesis'] = genesis_from_config_file
+
+    if bootstrap_nodes_from_config_file:
+        # Fixed bootstrap_nodes taken from profile must be deleted as custom bootstrap_nodes loaded
+        del config['discovery']['bootstrap_nodes']
+        config['discovery']['bootstrap_nodes'] = bootstrap_nodes_from_config_file
+
+    pre_cmd_line_config_genesis = config.get('eth', {}).get('genesis')
     # override values with values from cmd line
     for config_value in config_values:
         try:
             konfig.set_config_param(config, config_value)
-            # check if this is part of the default config
-            if config_value.startswith("eth.genesis"):
-                del config['eth']['genesis_hash']
         except ValueError:
             raise BadParameter('Config parameter must be of the form "a.b.c=d" where "a.b.c" '
                                'specifies the parameter to set and d is a valid yaml value '
                                '(example: "-c jsonrpc.port=5000")')
 
-    # Load genesis config
-    update_config_from_genesis_json(config, config['eth']['genesis'])
+    if pre_cmd_line_config_genesis != config.get('eth', {}).get('genesis'):
+        # Fixed genesis_hash taked from profile must be deleted as custom genesis loaded
+        if 'genesis_hash' in config['eth']:
+            del config['eth']['genesis_hash']
 
+    # Load genesis config
+    konfig.update_config_from_genesis_json(config,
+                                           genesis_json_filename_or_dict=config['eth']['genesis'])
     if bootstrap_node:
         config['discovery']['bootstrap_nodes'] = [bytes(bootstrap_node)]
     if mining_pct > 0:
@@ -134,7 +163,8 @@ def app(ctx, profile, alt_config, config_values, data_dir, log_config, bootstrap
 
 
 @app.command()
-@click.option('--dev/--nodev', default=False, help='Drop into interactive debugger on unhandled exceptions.')
+@click.option('--dev/--nodev', default=False,
+              help='Drop into interactive debugger on unhandled exceptions.')
 @click.option('--nodial/--dial',  default=False, help='Do not dial nodes.')
 @click.option('--fake/--nofake',  default=False, help='Fake genesis difficulty.')
 @click.option('--console',  is_flag=True, help='Immediately drop into interactive console.')
@@ -155,14 +185,15 @@ def run(ctx, dev, nodial, fake, console):
         blocks.GENESIS_DIFFICULTY = 1024
         blocks.BLOCK_DIFF_FACTOR = 16
         blocks.MIN_GAS_LIMIT = blocks.default_config['GENESIS_GAS_LIMIT'] / 2
+        config['eth']['block']['GENESIS_DIFFICULTY'] = 1024
+        config['eth']['block']['BLOCK_DIFF_FACTOR'] = 16
 
     # create app
     app = EthApp(config)
 
     # development mode
     if dev:
-        gevent.get_hub().SYSTEM_ERROR = BaseException
-        sys.excepthook = ultratb.VerboseTB(call_pdb=True, tb_offset=6)
+        enable_greenlet_debugger()
         try:
             config['client_version'] += '/' + os.getlogin()
         except:
@@ -177,7 +208,7 @@ def run(ctx, dev, nodial, fake, console):
         AccountsService.register_with_app(app)
         unlock_accounts(ctx.obj['unlock'], app.services.accounts, password=ctx.obj['password'])
         try:
-            coinbase = app.services.accounts.coinbase
+            app.services.accounts.coinbase
         except ValueError as e:
             log.fatal('invalid coinbase', coinbase=config.get('pow', {}).get('coinbase_hex'),
                       error=e.message)
@@ -186,7 +217,9 @@ def run(ctx, dev, nodial, fake, console):
     app.start_console = console
 
     # register services
-    for service in services:
+    contrib_services = load_contrib_services(config)
+
+    for service in services + contrib_services:
         assert issubclass(service, BaseService)
         if service.name not in app.config['deactivated_services'] + [AccountsService.name]:
             assert service.name not in app.services
