@@ -3,8 +3,8 @@ PROPAGATE_ERRORS = False
 
 ###############################
 
-import time
-from copy import copy, deepcopy
+import os
+from copy import deepcopy
 from decorator import decorator
 from collections import Iterable
 import inspect
@@ -25,18 +25,22 @@ from tinyrpc.exc import BadRequestError, MethodNotFoundError
 from tinyrpc.protocols.jsonrpc import JSONRPCProtocol, JSONRPCInvalidParamsError
 from tinyrpc.server.gevent import RPCServerGreenlets
 from tinyrpc.transports.wsgi import WsgiServerTransport
+from tinyrpc.transports import ServerTransport
 from devp2p.service import BaseService
 from eth_protocol import ETHProtocol
 from ethereum.trie import Trie
 from ethereum.utils import denoms
+import ethereum.bloom as bloom
+from accounts import Account
+from ipc_rpc import bind_unix_listener, serve
 
-from ethereum.utils import DEBUG
+from ethereum.utils import int32
 
 logger = log = slogging.get_logger('jsonrpc')
 
 # defaults
-default_startgas = 100 * 1000
-default_gasprice = 10 * denoms.szabo
+default_startgas = 500 * 1000
+default_gasprice = 60 * denoms.shannon
 
 
 def _fail_on_error_dispatch(self, request):
@@ -98,84 +102,64 @@ class LoggingDispatcher(RPCDispatcher):
         self.logger = log.debug
 
     def dispatch(self, request):
-        if isinstance(request, Iterable):
-            request_list = request
-        else:
-            request_list = [request]
-        for req in request_list:
-            self.logger('------------------------------')
-            self.logger('RPC call', method=req.method, args_=req.args, kwargs=req.kwargs,
-                        id=req.unique_id)
-        response = super(LoggingDispatcher, self).dispatch(request)
-        if isinstance(response, Iterable):
-            response_list = response
-        else:
-            response_list = [response]
-        for res in response_list:
-            if hasattr(res, 'result'):
-                self.logger('RPC result', id=res.unique_id, result=res.result)
+        try:
+            if isinstance(request, Iterable):
+                request_list = request
             else:
-                self.logger('RPC error', id=res.unique_id, error=res.error)
-        return response
+                request_list = [request]
+            for req in request_list:
+                self.logger('------------------------------')
+                self.logger('RPC call', method=req.method, args_=req.args, kwargs=req.kwargs,
+                            id=req.unique_id)
+            response = super(LoggingDispatcher, self).dispatch(request)
+            if isinstance(response, Iterable):
+                response_list = response
+            else:
+                response_list = [response]
+            for res in response_list:
+                if hasattr(res, 'result'):
+                    self.logger('RPC result', id=res.unique_id, result=res.result)
+                else:
+                    self.logger('RPC error', id=res.unique_id, error=res.error)
+            return response
+        except KeyError as e:
+            log.error("Unknown/unhandled RPC method!", method=e.args[0])
 
 
-class JSONRPCServer(BaseService):
-
-    """Service providing an HTTP server with JSON RPC interface.
-
-    Other services can extend the JSON RPC interface by creating a
-    :class:`Subdispatcher` and registering it via
-    `Subdispatcher.register(self.app.services.json_rpc_server)`.
-
-    Alternatively :attr:`dispatcher` can be extended directly (see
-    https://tinyrpc.readthedocs.org/en/latest/dispatch.html).
+class IPCDomainSocketTransport(ServerTransport):
+    """tinyrpc ServerTransport implementation for unix domain sockets.
     """
+    def __init__(self, sockpath=None, queue_class=gevent.queue.Queue):
+        self.socket = bind_unix_listener(sockpath)
+        self.messages = queue_class()
+        self.replies = queue_class()
 
-    name = 'jsonrpc'
-    default_config = dict(jsonrpc=dict(
-        listen_port=4000,
-        listen_host='127.0.0.1',
-        corsdomain=''))
+    def handle(self, socket, address):
+        while True:
+            try:
+                msg = socket.recv(4096)
+                if not msg:
+                    break
+                self.messages.put((socket, msg))
+                reply = self.replies.get()
+                socket.sendall(reply)
+            except IOError as e:
+                log.error("IOError on ipc socket", error=e.args)
+
+    def receive_message(self):
+        return self.messages.get()
+
+    def send_reply(self, context, reply):
+        self.replies.put(reply)
+
+
+class RPCServer(BaseService):
+    """Base-class for RPC-servers, can be extended for different transports.
+    """
 
     @classmethod
     def subdispatcher_classes(cls):
-        return (Web3, Net, Compilers, DB, Chain, Miner, FilterManager)
-
-    def __init__(self, app):
-        log.debug('initializing JSONRPCServer')
-        BaseService.__init__(self, app)
-        self.app = app
-
-        self.dispatcher = LoggingDispatcher()
-        # register sub dispatchers
-        for subdispatcher in self.subdispatcher_classes():
-            subdispatcher.register(self)
-
-        transport = WsgiServerTransport(queue_class=gevent.queue.Queue,
-                                        allow_origin=self.config['jsonrpc']['corsdomain'])
-        # start wsgi server as a background-greenlet
-        self.listen_port = self.config['jsonrpc']['listen_port']
-        self.listen_host = self.config['jsonrpc']['listen_host']
-        self.wsgi_server = gevent.wsgi.WSGIServer((self.listen_host, self.listen_port),
-                                                  transport.handle, log=WSGIServerLogger)
-        self.wsgi_thread = None
-        self.rpc_server = RPCServerGreenlets(
-            transport,
-            JSONRPCProtocol(),
-            self.dispatcher
-        )
-        self.default_block = 'latest'
-
-    def _run(self):
-        log.info('starting JSONRPCServer', port=self.listen_port)
-        # in the main greenlet, run our rpc_server
-        self.wsgi_thread = gevent.spawn(self.wsgi_server.serve_forever)
-        self.rpc_server.serve_forever()
-
-    def stop(self):
-        log.info('stopping JSONRPCServer')
-        if self.wsgi_thread is not None:
-            self.wsgi_thread.kill()
+        return (Web3, Personal, Net, Compilers, DB, Chain, Miner, FilterManager)
 
     def get_block(self, block_id=None):
         """Return the block identified by `block_id`.
@@ -192,6 +176,7 @@ class JSONRPCServer(BaseService):
         :returns: the requested block
         :raises: :exc:`KeyError` if the block does not exist
         """
+        log.debug("get_block")
         assert 'chain' in self.app.services
         chain = self.app.services.chain.chain
         if block_id is None:
@@ -216,6 +201,106 @@ class JSONRPCServer(BaseService):
         return chain.get(hash_)
 
 
+class IPCRPCServer(RPCServer):
+    """Service providing an IPC Service over a named socket.
+    Should respond to requests such as:
+
+        >>> echo '{"jsonrpc":"2.0","id":1,"method":"eth_getBlockByNumber","params":["latest",false]}' | socat - /tmp/geth.ipc
+    """
+    name = 'ipc'
+    default_config = dict(ipc=dict(
+        ipcpath='/tmp/pyethapp.ipc',
+    ))
+
+    def __init__(self, app):
+        log.debug('initializing IPCRPCServer')
+        BaseService.__init__(self, app)
+        self.app = app
+
+        self.dispatcher = LoggingDispatcher()
+        # register sub dispatchers
+        for subdispatcher in self.subdispatcher_classes():
+            subdispatcher.register(self)
+
+        self.ipcpath = self.config['ipc']['ipcpath']
+        self.transport = IPCDomainSocketTransport(queue_class=gevent.queue.Queue,
+                sockpath=self.ipcpath)
+
+        self.rpc_server = RPCServerGreenlets(
+            self.transport,
+            JSONRPCProtocol(),
+            self.dispatcher
+        )
+        self.default_block = 'latest'
+
+    def _run(self):
+        log.info('starting IPCRPCServer', ipcpath=self.ipcpath)
+        # in the main greenlet, run our rpc_server
+        self.socket_server = gevent.spawn(serve, self.transport.socket, handler=self.transport.handle)
+        self.rpc_server.serve_forever()
+
+    def stop(self):
+        log.info('stopping IPCRPCServer')
+        if self.socket_server is not None:
+            self.socket_server.kill()
+
+
+class JSONRPCServer(RPCServer):
+    """Service providing an HTTP server with JSON RPC interface.
+
+    Other services can extend the JSON RPC interface by creating a
+    :class:`Subdispatcher` and registering it via
+    `Subdispatcher.register(self.app.services.json_rpc_server)`.
+
+    Alternatively :attr:`dispatcher` can be extended directly (see
+    https://tinyrpc.readthedocs.org/en/latest/dispatch.html).
+    """
+
+    name = 'jsonrpc'
+    default_config = dict(jsonrpc=dict(
+        listen_port=4000,
+        listen_host='127.0.0.1',
+        corsdomain='',
+        ))
+
+    def __init__(self, app):
+        log.debug('initializing JSONRPCServer')
+        BaseService.__init__(self, app)
+        self.app = app
+
+        self.dispatcher = LoggingDispatcher()
+        # register sub dispatchers
+        for subdispatcher in self.subdispatcher_classes():
+            subdispatcher.register(self)
+
+        transport = WsgiServerTransport(queue_class=gevent.queue.Queue,
+                                        allow_origin=self.config['jsonrpc']['corsdomain'])
+        # start wsgi server as a background-greenlet
+        self.listen_port = self.config['jsonrpc']['listen_port']
+        self.listen_host = self.config['jsonrpc']['listen_host']
+        listener = (self.listen_host, self.listen_port)
+        self.wsgi_server = gevent.wsgi.WSGIServer(listener,
+                                                  transport.handle, log=WSGIServerLogger)
+        self.wsgi_thread = None
+        self.rpc_server = RPCServerGreenlets(
+            transport,
+            JSONRPCProtocol(),
+            self.dispatcher
+        )
+        self.default_block = 'latest'
+
+    def _run(self):
+        log.info('starting JSONRPCServer', port=self.listen_port)
+        # in the main greenlet, run our rpc_server
+        self.wsgi_thread = gevent.spawn(self.wsgi_server.serve_forever)
+        self.rpc_server.serve_forever()
+
+    def stop(self):
+        log.info('stopping JSONRPCServer')
+        if self.wsgi_thread is not None:
+            self.wsgi_thread.kill()
+
+
 class Subdispatcher(object):
 
     """A JSON RPC subdispatcher which can be registered at JSONRPCService.
@@ -514,6 +599,41 @@ def encode_res(encoder):
     return new_f
 
 
+class Personal(Subdispatcher):
+
+    """Subdispatcher for account-related RPC methods.
+
+    NOTE: this does not seem to be part of the official JSON-RPC specs but instead part of
+    go-ethereum's JavaScript-Console: https://github.com/ethereum/go-ethereum/wiki/JavaScript-Console#personal
+
+    It is needed for MIST-IPC.
+    """
+
+    prefix = 'personal_'
+
+    @public
+    @decode_arg('account_address', address_decoder)
+    def unlockAccount(self, account_address, passwd, duration):
+        if account_address in self.app.services.accounts:
+            account = self.app.services.accounts.get_by_address(account_address)
+            account.unlock(passwd)
+            gevent.spawn_later(duration, lambda: account.lock())
+            return not account.locked
+        else:
+            return False
+
+    @public
+    @encode_res(address_encoder)
+    def newAccount(self, passwd):
+        account = Account.new(passwd)
+        account.path = os.path.join(self.app.services.accounts.keystore_dir, account.address.encode('hex'))
+        self.app.services.accounts.add_account(account)
+        account.lock()
+        assert account.locked
+        assert self.app.services.accounts.find(account.address.encode('hex'))
+        return account.address
+
+
 class Web3(Subdispatcher):
 
     """Subdispatcher for some generic RPC methods."""
@@ -528,7 +648,7 @@ class Web3(Subdispatcher):
 
     @public
     def clientVersion(self):
-        return self.app.client_version
+        return "{s.app.client_name}/{s.app.client_version}".format(s=self)
 
 
 class Net(Subdispatcher):
@@ -691,6 +811,19 @@ class Chain(Subdispatcher):
     def protocolVersion(self):
         return str(ETHProtocol.version)
 
+    @public
+    def syncing(self):
+        if not self.chain.is_syncing:
+            return False
+        else:
+            synctask = self.chain.synchronizer.synctask
+            result = dict(
+                startingBlock=synctask.start_block_number,
+                currentBlock=self.chain.chain.head.number,
+                highestBlock=synctask.end_block_number,
+            )
+            return {k: quantity_encoder(v) for k, v in result.items()}
+
     @public
     @encode_res(quantity_encoder)
     def blockNumber(self):
@@ -720,7 +853,8 @@ class Chain(Subdispatcher):
     @encode_res(quantity_encoder)
     def getTransactionCount(self, address, block_id='pending'):
         block = self.json_rpc_server.get_block(block_id)
-        return block.get_nonce(address)
+        return block.get_nonce(address) - \
+            self.json_rpc_server.config['eth']['block']['ACCOUNT_INITIAL_NONCE']
 
     @public
     @decode_arg('block_hash', block_hash_decoder)
@@ -756,6 +890,8 @@ class Chain(Subdispatcher):
     @decode_arg('block_id', block_id_decoder)
     def getUncleCountByBlockNumber(self, block_id):
         try:
+            if block_id == u'pending':
+                return None
             block = self.json_rpc_server.get_block(block_id)
         except KeyError:
             return None
@@ -795,9 +931,11 @@ class Chain(Subdispatcher):
     def getTransactionByHash(self, tx_hash):
         try:
             tx, block, index = self.chain.chain.index.get_transaction(tx_hash)
+            if self.chain.chain.in_main_branch(block):
+                return tx_encoder(tx, block, index, False)
         except KeyError:
             return None
-        return tx_encoder(tx, block, index, False)
+        return None
 
     @public
     @decode_arg('block_hash', block_hash_decoder)
@@ -837,6 +975,9 @@ class Chain(Subdispatcher):
     @decode_arg('index', quantity_decoder)
     def getUncleByBlockNumberAndIndex(self, block_id, index):
         try:
+            #TODO: think about moving this check into the Block.uncles property
+            if block_id == u'pending':
+                return None
             block = self.json_rpc_server.get_block(block_id)
             uncle = block.uncles[index]
         except (IndexError, KeyError):
@@ -850,7 +991,7 @@ class Chain(Subdispatcher):
         return [
             encode_hex(h.header.mining_hash),
             encode_hex(h.header.seed),
-            encode_hex(zpad(int_to_big_endian(2**256 // h.header.difficulty), 32))
+            encode_hex(zpad(int_to_big_endian(2 ** 256 // h.header.difficulty), 32))
         ]
 
     @public
@@ -901,6 +1042,24 @@ class Chain(Subdispatcher):
     def coinbase(self):
         return self.app.services.accounts.coinbase
 
+    @public
+    @decode_arg('address', address_decoder)
+    @decode_arg('block_id', block_id_decoder)
+    @encode_res(quantity_encoder)
+    def nonce(self, address, block_id='pending'):
+        """
+        Return the `nonce` for `address` at the current `block_id
+        :param address:
+        :param block_id:
+        :return: the next nonce for the address/account
+        """
+        assert address is not None
+        block = self.json_rpc_server.get_block(block_id)
+        assert block is not None
+        nonce = block.get_nonce(address)
+        assert nonce is not None and isinstance(nonce, int)
+        return nonce
+
     @public
     def sendTransaction(self, data):
         """
@@ -933,19 +1092,46 @@ class Chain(Subdispatcher):
             assert nonce is not None, 'signed but no nonce provided'
             assert v and r and s
         else:
-            nonce = self.app.services.chain.chain.head_candidate.get_nonce(sender)
+            if nonce is None or nonce == 0:
+                nonce = self.app.services.chain.chain.head_candidate.get_nonce(sender)
 
         tx = Transaction(nonce, gasprice, startgas, to, value, data_, v, r, s)
         tx._sender = None
-        print tx.log_dict()
         if not signed:
-            assert sender in self.app.services.accounts, 'no account for sender'
+            assert sender in self.app.services.accounts, 'can not sign: no account for sender'
             self.app.services.accounts.sign_tx(sender, tx)
-        self.app.services.chain.add_transaction(tx, origin=None)
-
+        self.app.services.chain.add_transaction(tx, origin=None, force_broadcast=True)
         log.debug('decoded tx', tx=tx.log_dict())
         return data_encoder(tx.hash)
 
+    @public
+    @decode_arg('data', data_decoder)
+    def sendRawTransaction(self, data):
+        """
+        decode sendRawTransaction request data, format it and relay along to the sendTransaction method
+        to ensure the same validations and processing rules are applied
+        """
+        tx_data = rlp.codec.decode(data, ethereum.transactions.Transaction)
+
+        tx_dict = tx_data.to_dict()
+        # encode addresses
+        tx_dict['from'] = address_encoder(tx_dict.get('sender', ''))
+        to_value = tx_dict.get('to', '')
+        if to_value:
+            tx_dict['to'] = address_encoder(to_value)
+
+        # encode data
+        tx_dict['data'] = data_encoder(tx_dict.get('data', b''))
+
+        # encode quantities included in the raw transaction data
+        gasprice_key = 'gasPrice' if 'gasPrice' in tx_dict else 'gasprice'
+        gas_key = 'gas' if 'gas' in tx_dict else 'startgas'
+        for key in ('value', 'nonce', gas_key, gasprice_key, 'v', 'r', 's'):
+            if key in tx_dict:
+                tx_dict[key] = quantity_encoder(tx_dict[key])
+
+        # relay the information along to the sendTransaction method for processing
+        return self.sendTransaction(tx_dict)
 
     @public
     @decode_arg('block_id', block_id_decoder)
@@ -964,11 +1150,12 @@ class Chain(Subdispatcher):
                 success, output = processblock.apply_transaction(test_block, tx)
                 assert success
         else:
-            test_block = ethereum.blocks.genesis(block.db)
+            env = ethereum.config.Env(db=block.db)
+            test_block = ethereum.blocks.genesis(env)
             original = {key: value for key, value in snapshot_before.items() if key != 'txs'}
             original = deepcopy(original)
             original['txs'] = Trie(snapshot_before['txs'].db, snapshot_before['txs'].root_hash)
-            test_block = ethereum.blocks.genesis(block.db)
+            test_block = ethereum.blocks.genesis(env)
             test_block.revert(original)
 
         # validate transaction
@@ -978,7 +1165,7 @@ class Chain(Subdispatcher):
         try:
             startgas = quantity_decoder(data['gas'])
         except KeyError:
-            startgas = block.gas_limit - block.gas_used
+            startgas = test_block.gas_limit - test_block.gas_used
         try:
             gasprice = quantity_decoder(data['gasPrice'])
         except KeyError:
@@ -1003,7 +1190,7 @@ class Chain(Subdispatcher):
 
         try:
             success, output = processblock.apply_transaction(test_block, tx)
-        except processblock.InvalidTransaction as e:
+        except processblock.InvalidTransaction:
             success = False
         # make sure we didn't change the real state
         snapshot_after = block.snapshot()
@@ -1071,7 +1258,7 @@ class Chain(Subdispatcher):
 
         try:
             success, output = processblock.apply_transaction(test_block, tx)
-        except processblock.InvalidTransaction as e:
+        except processblock.InvalidTransaction:
             success = False
         # make sure we didn't change the real state
         snapshot_after = block.snapshot()
@@ -1146,22 +1333,42 @@ class LogFilter(object):
 
         # skip blocks that have already been checked
         if self.last_block_checked is not None:
+            print self.last_block_checked
             first = max(self.last_block_checked.number + 1, first)
             if first > last:
                 return {}
 
-        blocks_to_check = [self.chain.get(self.chain.index.get_block_by_number(n))
-                           for n in range(first, last)]
+        blocks_to_check = []
+        for n in range(first, last):
+            blocks_to_check.append(self.chain.index.get_block_by_number(n))
         # last block may be head candidate, which cannot be retrieved via get_block_by_number
         if last == self.chain.head_candidate.number:
             blocks_to_check.append(self.chain.head_candidate)
         else:
             blocks_to_check.append(self.chain.get(self.chain.index.get_block_by_number(last)))
+        logger.debug('obtained block hashes to check with filter', numhashes=len(blocks_to_check))
 
         # go through all receipts of all blocks
-        logger.debug('blocks to check', blocks=blocks_to_check)
+        # logger.debug('blocks to check', blocks=blocks_to_check)
         new_logs = {}
-        for block in blocks_to_check:
+        for i, block in enumerate(blocks_to_check):
+            if not isinstance(block, (ethereum.blocks.Block, ethereum.blocks.CachedBlock)):
+                _bloom = self.chain.get_bloom(block)
+                # Check that the bloom for this block contains at least one of the desired
+                # addresses
+                if self.addresses:
+                    pass_address_check = False
+                    for address in self.addresses:
+                        if bloom.bloom_query(_bloom, address):
+                            pass_address_check = True
+                    if not pass_address_check:
+                        continue
+                # Check that the bloom for this block contains all of the desired topics
+                _topic_bloom = bloom.bloom_from_list(map(int32.serialize, self.topics or []))
+                if bloom.bloom_combine(_bloom, _topic_bloom) != _bloom:
+                    continue
+                block = self.chain.get(block)
+                print 'bloom filter passed'
             logger.debug('-')
             logger.debug('with block', block=block)
             receipts = block.get_receipts()
@@ -1197,6 +1404,8 @@ class LogFilter(object):
             self.last_block_checked = blocks_to_check[-1]
         else:
             self.last_block_checked = blocks_to_check[-2] if len(blocks_to_check) >= 2 else None
+        if self.last_block_checked and not isinstance(self.last_block_checked, (ethereum.blocks.Block, ethereum.blocks.CachedBlock)):
+            self.last_block_checked = self.chain.get(self.last_block_checked)
         actually_new_ids = new_logs.viewkeys() - self.log_dict.viewkeys()
         self.log_dict.update(new_logs)
         return {id_: new_logs[id_] for id_ in actually_new_ids}
@@ -1318,7 +1527,7 @@ class FilterManager(Subdispatcher):
     @decode_arg('id_', quantity_decoder)
     def uninstallFilter(self, id_):
         try:
-            f = self.filters.pop(id_)
+            self.filters.pop(id_)
             return True
         except KeyError:
             return False
@@ -1354,7 +1563,6 @@ class FilterManager(Subdispatcher):
         filter_ = filter_decoder(filter_dict, self.chain.chain)
         return filter_.logs
 
-
     # ########### Trace ############
     def _get_block_before_tx(self, txhash):
         tx, blk, i = self.app.services.chain.chain.index.get_transaction(txhash)
@@ -1418,6 +1626,8 @@ class FilterManager(Subdispatcher):
             tx, block, index = self.chain.chain.index.get_transaction(tx_hash)
         except KeyError:
             return None
+        if not self.chain.chain.in_main_branch(block):
+            return None
         receipt = block.get_receipt(index)
         response = {
             'transactionHash': data_encoder(tx.hash),
@@ -1449,7 +1659,6 @@ class FilterManager(Subdispatcher):
 
 
 if __name__ == '__main__':
-    import inspect
     from devp2p.app import BaseApp
 
     # deactivate service availability check
